# Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient import base


class FloatingIPPool(base.Resource):
    def __repr__(self):
        return "<FloatingIPPool: name=%s>" % self.name


class FloatingIPPoolManager(base.ManagerWithFind):
    resource_class = FloatingIPPool

    def list(self):
        """Retrieve a list of all floating ip pools."""
        return self._list('/os-floating-ip-pools', 'floating_ip_pools')
